/*
 * Copyright (c) 2008 Imago Scientific Instruments Corp.
 *
 * This program is the property of Imago Scientific Instruments Corp. and may not be
 * distributed in any form, in whole or in part, without permission from Imago.
 *
 */
#pragma warning (disable : 4800)
#include <stdio.h>
#include <assert.h>
// A legal ini File has format:
//[sectionname1]
//varname1= value
//varname2= value
//[sectionname2]
//varname1= value
// 
// no space allowed before '='

// ROOT includes
#include "TString.h"
#include "TSystem.h"
//project include files
#include "CIniFile.h"
int CIniFile::fPrintError=0;
#if !defined(__CINT__)  || defined(__MAKECINT__)
ClassImp(CIniFile)
#endif
Bool_t CIniFile::fServerMode = kFALSE;
//_____________________________________________________________________________
CIniFile::CIniFile(const TString& filename,Bool_t bQuiet) 
{
	// Constructor, that opens or creates an ini-file
	// open file
	fFileName = new TString(filename);
	if (fServerMode && filename.CompareTo("leap.ini",TString::kIgnoreCase)==0)
	{
		// if server mode, make the file unique.
		//*fFileName = Form("leap_%d.ini",gSystem->GetPid());
		delete fFileName;
		fFileName = new TString(Form("leap_%d.ini",gSystem->GetPid()));
	}
	fStr=0;
	fQuiet = bQuiet;
}
//_____________________________________________________________________________
CIniFile::~CIniFile()
{
	if (fFileName) delete fFileName;
	if (fStr) delete fStr;
}
//_____________________________________________________________________________
int CIniFile::GetSection(const TString& Section, int& spos, int& sn, int& pos, int& n)
{
	// Get section from filestring
	// spos is position of section start, the "[" char
	// sn total section length, including [sect]
	// pos is position of first char after the section's "]"
	// n # of chars in the section, not including [sect]
	TString BrackSectStr(Section);
	BrackSectStr.Prepend("[");
	BrackSectStr.Append("]");

	int BrackSectStrPos=0;

	BrackSectStrPos=fStr->Index(BrackSectStr);
	if(BrackSectStrPos<0)
	{
		// Not able to find section: Add to end of file
		fStr->Append(BrackSectStr.Data());
		fStr->Append("\n");
		BrackSectStrPos = fStr->Index(BrackSectStr);
		fChanged=kTRUE;
	}
	spos=BrackSectStrPos;
	pos=BrackSectStrPos+(BrackSectStr.Length());
	int SectStopPos=fStr->Index("[",pos)-1;
	if(SectStopPos<0) SectStopPos=fStr->Length()-1;  
	n=(SectStopPos-pos)+1;
	sn=(SectStopPos-spos)+1;
	return 0;
}
//_____________________________________________________________________________
int CIniFile::GetSection(const TString& Section, int& spos, int& sn)
{
	int pos=0; int n=0;
	return GetSection(Section, spos, sn, pos, n);
}
//_____________________________________________________________________________
Bool_t CIniFile::IsVarInSect(const TString &Section,const TString &Variable)
{
	// return kTRUE if Section exists and Variable is in that section
	TString BrackSectStr(Section);
	BrackSectStr.Prepend("[");
	BrackSectStr.Append("]");

	int BrackSectStrPos=0;

	BrackSectStrPos=fStr->Index(BrackSectStr);
	if(BrackSectStrPos<0) return kFALSE;		// Section does not exist!

	// get section position and length
	int SectPos=0;
	int SectLen=0;
	GetVarAreaInSection(Section,SectPos,SectLen); 
	TString VarStr(Variable);
	VarStr.Append("=");
	int VarStartPos=fStr->Index(VarStr,SectPos);
	if ( VarStartPos<0 || 
		 SectLen<0 || 
		 VarStartPos>SectPos+SectLen-1)
		return kFALSE;
	return kTRUE;
}
//_____________________________________________________________________________
int CIniFile::GetVarAreaInSection(const TString& Section, int& pos, int& n)
{
	int spos=0; int sn=0;
	return GetSection(Section, spos, sn, pos, n);
}
//_____________________________________________________________________________
int CIniFile::GetVarInSect(const TString& Section, const TString& Variable, int& lpos, int& ln ,int& pos, int& n)
{
	// Get variable index from section in filestring
	// lpos= first character of variable name
	// ln  = number of characters of name=value string
	// pos = first character of variable value
	// n   = number of characters in variable value
	int SectPos=0;
	int SectLen=0;
	if(GetVarAreaInSection(Section,SectPos,SectLen)<0) 
	{
		lpos=0;ln=0;pos=0;n=0;return -1;
	}
	TString VarStr(Variable);
	VarStr.Append("=");
	//int lenvar = VarStr.Length();
	//if (lenvar<30) VarStr.Resize(30);
	int VarStartPos=fStr->Index(VarStr,SectPos);

	// ECR30206
	bool found = false;
	if( SectLen>0 )
	{
		while ((!found) && (VarStartPos<=(SectPos+SectLen-1) && (VarStartPos>=0))  )
		{
			// Check that we have the exact string
			const char prevChar = *(fStr->Data() + VarStartPos-1);
			if((prevChar == '\n') || (prevChar == ' ')) 
			{
				found = true;
			}
			else
			{	
				VarStartPos=fStr->Index(VarStr,VarStartPos+1);
			}
		}
	}

	if(!found)
	{
		// Can't find variable in section - need to write it.
		const char last = *(fStr->Data() + SectPos + SectLen -1);
		if (last!='\n') 
		{
			// last character in section is not a line end
			fStr->Insert(SectPos+SectLen,"\n");
			SectLen++;
		}
		fStr->Insert(SectPos+SectLen,VarStr);
		VarStartPos=SectPos+SectLen;
		SectLen += VarStr.Length();
		fChanged = kTRUE;
	}
	lpos=VarStartPos;
	pos=VarStartPos+VarStr.Length();
	int newline=fStr->Index("\n",pos);
	if (newline<0 || newline>SectPos+SectLen)
	{
		fStr->Insert(SectPos+SectLen,"\n");
		newline = fStr->Index("\n",pos);
	}
	ln=(newline-VarStartPos)+1;
	n=((newline-1)-pos)+1;
	return 0;
}
//_____________________________________________________________________________
int CIniFile::GetVarDataInSect(const TString& Section, const TString& Variable, int& pos, int& n)
{
	int lpos=0; int ln=0;
	return GetVarInSect(Section,Variable,lpos,ln,pos,n);
}
//_____________________________________________________________________________
int CIniFile::GetVarLineInSect(const TString& Section, const TString& Variable, int& lpos, int& ln)
{
	int pos=0; int n=0;
	return GetVarInSect(Section,Variable,lpos,ln,pos,n);
}
//_____________________________________________________________________________
int CIniFile::DeleteSection(const TString& Section)
{
	// remove section with all its variables
	int pos=0; int n=0;
	int err=GetSection(Section,pos,n);
	if(err<0) return err;
	fStr->Remove(pos,n+1);
	return 0;
}
//_____________________________________________________________________________
int CIniFile::DeleteVarInSect(const TString& Section, const TString& Variable)
{
	// remove Variable in Section
	int lpos=0; int ln=0;
	int err= GetVarLineInSect(Section,Variable,lpos,ln);
	if(err<0) return err;
	fStr->Remove(lpos,ln);
	return 0;
}
//_____________________________________________________________________________
void CIniFile::Read(const TString& Section, const TString& Variable, const TString& Default, TString& Result)
{
	// read methods
	// 1. TStrings
	ReadIni();
	int VarPos=0;
	int VarLen=0;
	int err=GetVarDataInSect(Section,Variable,VarPos,VarLen);
	assert(!err);
	 // Extract substring
	if (VarLen==0)
	{
		fStr->Insert(VarPos,Default);
		Result=Default;
		fChanged = kTRUE;
	}
	else
	{
		const char *rel = fStr->Data()+VarPos;
		Result.Replace(0,Result.Length(),rel,VarLen);
		while (*Result.Data()==' ') Result.Remove(0,1);
	}
	if (fChanged) 
	{
		if (RegenerateIniFile())
		{
			if (!fQuiet)printf("CIniFile::Read(): %s[%s]%s=%s \n",
				fFileName->Data(),Section.Data(),Variable.Data(),Default.Data());
		}
		else
		{
			printf("CIniFile::Read(): Error writing default [%s]%s=%s\n",
				Section.Data(),Variable.Data(),Default.Data());
		}
	}
	if (fStr) delete fStr;
	fStr = 0;
}
//_____________________________________________________________________________
Bool_t CIniFile::RegenerateIniFile()
{
	// Write out .ini file.  Note: replacing \n with \r\n is to put windows endings
	// on the ascii file.  For unix, do nothing; for Mac, replace \n with \r
	FILE *fp = fopen(fFileName->Data(),"wb");
	if (!fp)
	{
		printf("CIniFile::RegenerateIniFile(): could open %s for writing\n",fFileName->Data());
		return kFALSE;
	}
	fStr->ReplaceAll("\n",1,"\r\n",2);
	fwrite(fStr->Data(),fStr->Length(),1,fp);
	fclose(fp);
	fStr->ReplaceAll("\r\n",2,"\n",1);
	fChanged=kFALSE;
	return kTRUE;
}
//_____________________________________________________________________________
void CIniFile::Read(const TString& Section, const TString& Variable, const int Default, int& Result)
{
	// 2. integers, booleans
	TString ResultStr;
	char *DefaultText = new char[20];
	sprintf(DefaultText,"%d",Default); 
	Read(Section, Variable, DefaultText, ResultStr);
	Result = atoi(ResultStr);
	delete [] DefaultText;
}
//_____________________________________________________________________________
void CIniFile::Read(const TString& Section, const TString& Variable, const float Default, float& Result)
{
	// 4. floats
	TString ResultStr;
	char *DefaultText = new char[30];
	sprintf(DefaultText,"%10.5g",Default); 
	Read(Section, Variable, DefaultText, ResultStr);
	Result = (float)atof(ResultStr);
	delete [] DefaultText;
}
//_____________________________________________________________________________
void CIniFile::Read(const TString& Section, const TString& Variable, const double Default, double& Result)
{
	// 5. doubles
	TString ResultStr;
	char *DefaultText = new char[40];
	sprintf(DefaultText,"%10.5g",Default); 
	Read(Section, Variable, DefaultText, ResultStr);
	Result = atof(ResultStr);
	delete [] DefaultText;
}
//_____________________________________________________________________________
void CIniFile::Write(const TString& Section, const TString& Variable, const TString& Value)
{
	// write methods
	// 1. TStrings
	ReadIni();
	int pos=0; int n=0;
	TString VarStr(Variable);
	VarStr.Append("=");
 
	int err=GetVarDataInSect(Section,Variable,pos,n);
	fStr->Replace(pos,n,Value,Value.Length());
	if (!RegenerateIniFile())
	{
		printf("CIniFile::Write(): Error writing [%s]%s=%s\n",Section.Data(),Variable.Data(),Value.Data());
	}
	return;
}
//_____________________________________________________________________________
void CIniFile::Write(const TString& Section, const TString& Variable, const int Value)
{
	// 2. integers, booleans
	char *ValueText = new char[20];
	sprintf(ValueText,"%d",Value); 
	Write(Section, Variable, ValueText);
	delete [] ValueText;
}

//_____________________________________________________________________________
void CIniFile::Write(const TString& Section, const TString& Variable, const float Value)
{
	// 4. floats
	char *ValueText = new char[30];
	sprintf(ValueText,"%10.5g",Value); 
	Write(Section, Variable, ValueText);
	delete [] ValueText;
}
//_____________________________________________________________________________
void CIniFile::Write(const TString& Section, const TString& Variable, const double Value)
{
	// 5. double
	char *ValueText = new char[30];
	sprintf(ValueText,"%10.5g",Value); 
	Write(Section, Variable, ValueText);
	delete [] ValueText;
}
//_____________________________________________________________________________
void CIniFile::SetString(TString &result,const TString& Variable, const TString& Value)
{
	// setString methods
	// 1. TStrings
	result=(char)0;
	result.Append(Variable);
	result.Append("=");
	result.Append(Value);
	result.Append("\r\n");
 
	return;
}
//_____________________________________________________________________________
void CIniFile::SetString(TString &result,const TString& Variable, const int Value)
{
	// setString methods
	// 2. int
	result=(char)0;
	result.Append(Variable);
	result.Append("=");
	char *ValueText = new char[20];
	sprintf(ValueText,"%d",Value); 
	result.Append(ValueText);
	result.Append("\r\n");
 	delete [] ValueText;

	return;
}
//_____________________________________________________________________________
void CIniFile::SetString(TString &result,const TString& Variable, const float Value)
{
	// setString methods
	// 3. float
	result=(char)0;
	result.Append(Variable);
	result.Append("=");
	char *ValueText = new char[30];
	sprintf(ValueText,"%10.5g",Value); 
	result.Append(ValueText);
	result.Append("\r\n");
 	delete [] ValueText;

	return;
}
//_____________________________________________________________________________
void CIniFile::SetString(TString &result,const TString& Variable, const double Value)
{
	// setString methods
	// 4. double
	result=(char)0;
	result.Append(Variable);
	result.Append("=");
	char *ValueText = new char[30];
	sprintf(ValueText,"%10.5g",Value); 
	result.Append(ValueText);
	result.Append("\r\n");
 	delete [] ValueText;

	return;
}
//_____________________________________________________________________________
void CIniFile::SetSectionString(TString& Result,const TString& Section)
{
	Result.Form("[%s]\r\n",Section.Data());
}
//_____________________________________________________________________________
void CIniFile::ReadIni()
{
	fStr =  0;
	FILE *fp = fopen(fFileName->Data(),"rb");
	if (fp)
	{
		char *fs= new char[655360];
		int n=0;
		if (fp) 
			n = fread(fs,1,655360,fp);
		fs[n]=0;
		fStr = new TString(fs);
		delete [] fs;
		fclose(fp);
		fStr->ReplaceAll("\r\n",2,"\n",1);
	}
	else
	{
		if (!fQuiet) printf("CIniFile::CIniFile() : Creating %s\n",fFileName->Data());
		fStr = new TString("");
	}
	fChanged = kFALSE;
}