/*
 * Copyright (c) 2008 Imago Scientific Instruments Corp.
 *
 * This program is the property of Imago Scientific Instruments Corp. and may not be
 * distributed in any form, in whole or in part, without permission from Imago.
 *
 */
#ifndef _included_inifile_
#define _included_inifile_
#pragma warning( disable : 4275 )
#include "TString.h"
#include "TObject.h"
#if defined(__CINT__)
class CIniFile : public TObject
#else
#ifdef ROOT_EXPORTS
#define ROOT_API __declspec(dllexport)
#else
#define ROOT_API __declspec(dllimport)
#endif
class ROOT_API CIniFile : public TObject
#endif
{
private:
	TString *fFileName; //!
	TString *fStr;    //!
	Bool_t fChanged;
	Bool_t fQuiet;
	static int fPrintError;
	Bool_t RegenerateIniFile();
	static Bool_t fServerMode;
public:
	static void SetServerMode(Bool_t mode=kTRUE) {fServerMode = mode;}
	// constructor: opens an ini-file
	CIniFile(const TString& filename,Bool_t bQuiet=kFALSE);
	// destructor, closes file and cleans up
	~CIniFile();
	// 2 helpful methods
	// Get section from filestring
	int GetSection(const TString& Section, int& spos, int& sn, int& pos, int& n);
	int GetSection(const TString& Section, int& spos, int& sn);
	int GetVarAreaInSection(const TString& Section, int& pos, int& n);
	// Get variable from section in filestring
	Bool_t IsVarInSect(const TString& Section, const TString& Variable);
	int GetVarInSect(const TString& Section, const TString& Variable, int& lpos, int& ln ,int& pos, int& n);
	int GetVarDataInSect(const TString& Section, const TString& Variable, int& pos, int& n);
	int GetVarLineInSect(const TString& Section, const TString& Variable, int& lpos, int& ln);
	// remove section with all its variables
	int CIniFile::DeleteSection(const TString& Section);
	// remove variable in Section
	int CIniFile::DeleteVarInSect(const TString& Section, const TString& Variable);
	// read methods
	// 1. strings
	void Read(const TString& Section, const TString& Variable, const TString& Default, TString& Result);
	// 2. integers, booleans
	void Read(const TString& Section, const TString& Variable, const int Default, int& Result);
	// 4. floats
	void Read(const TString& Section, const TString& Variable, const float Default, float& Result);
	// 5. doubles
	void Read(const TString& Section, const TString& Variable, const double Default, double& Result);
	// write methods
	// 1. strings
	void Write(const TString& Section, const TString& Variable, const TString& Value);
	// 2. integers, booleans
	void Write(const TString& Section, const TString& Variable, const int Value);
	// 4. floats
	void Write(const TString& Section, const TString& Variable, const float Value);
	// 5. double
	void Write(const TString& Section, const TString& Variable, const double Value);
	// setString methods
	// 1. strings
	static void SetString(TString& Result,const TString& Variable, const TString& Value);
	// 2. integers, booleans
	static void SetString(TString& Result,const TString& Variable, const int Value);
	// 4. floats
	static void SetString(TString& Result,const TString& Variable, const float Value);
	// 5. double
	static void SetString(TString& Result,const TString& Variable, const double Value);
	static void SetSectionString(TString& Result,const TString& Section);
	void ReadIni();

#if !defined(__CINT__)  || defined(__MAKECINT__)
	ClassDef(CIniFile,0)
#endif  
};

#endif
